const MongoClient = require("mongodb").MongoClient;
const faker = require("faker");
const seedrandom = require("seedrandom");

faker.seed(123);
const random = seedrandom(123);

const randomIntFromInterval = (start, end) => {
  return Math.floor(random() * end) + start;
};

// const cleverCloud =
//   "mongodb://ulan5mpfqkpmeu9ksghd:VXkXCSsnf5g54p1sarqk@b8zzhjhm2dcvxs1-mongodb.services.clever-cloud.com:27017/b8zzhjhm2dcvxs1";
// const dbName = "b8zzhjhm2dcvxs1";

const generateData = async multiplier => {
  const dbName = "dipl";
  const client = new MongoClient("mongodb://localhost:27017", {
    useNewUrlParser: true
  });

  const cityAddress = [];
  const orderDates = [];
  const usersCount = 1000 * multiplier;
  const productsCount = 10000 * multiplier;
  const warehousesCount = 10 * multiplier;
  const ordersCount = 5000 * multiplier;
  const driveRecordsCount = 500 * multiplier;
  const cityAddressCount = 10 * multiplier;
  const overDateCount = 50 * multiplier;
  const maxItemsInOrderCount = 50 * multiplier;

  for (let i = 0; i < cityAddressCount; i++) {
    cityAddress.push([faker.address.city(), faker.address.zipCode()]);
  }

  for (let i = 0; i < driveRecordsCount; i++) {
    orderDates.push(faker.date.past());
  }
  try {
    // console.log("nosql - creating connection");
    await client.connect();
    // console.log("nosql - dropping database");
    await client.db(dbName).dropDatabase();
    const db = client.db(dbName);

    // console.log("nosql - creating users");
    const users = [];
    for (let i = 0; i < usersCount; i++) {
      const cA = cityAddress[randomIntFromInterval(0, cityAddressCount)];
      users.push({
        _id: "customer" + i,
        city: cA[0],
        zip_code: cA[1],
        street: faker.address.streetName()
      });
    }
    await db.collection("user").insertMany(users);

    // console.log("nosql - creating products");
    const products = [];
    for (let i = 0; i < productsCount; i++) {
      products.push({
        _id: "product" + i,
        price: randomIntFromInterval(100, 10000),
        profit_margin: randomIntFromInterval(1, 20)
      });
    }
    await db.collection("product").insertMany(products);

    // console.log("nosql - creating warehouses");
    const warehouses = [];
    for (let i = 0; i < warehousesCount; i++) {
      const cA = cityAddress[randomIntFromInterval(0, cityAddressCount)];
      warehouses.push({
        _id: "warehouse" + i,
        city: cA[0],
        zip_code: cA[1],
        street: faker.address.streetName()
      });
    }
    await db.collection("warehouse").insertMany(warehouses);

    // console.log("nosql - creating orders with products");
    for (let i = 0; i < ordersCount; i++) {
      const productIds = [];
      for (let j = 0; j < randomIntFromInterval(1, maxItemsInOrderCount); j++) {
        productIds.push("product" + randomIntFromInterval(0, productsCount));
      }
      const userId = randomIntFromInterval(0, usersCount);
      let date =
        orderDates[randomIntFromInterval(0, driveRecordsCount + overDateCount)];
      if (!date) {
        date = faker.date.future();
      }

      await db.collection("order").insertOne({
        _id: "order" + i,
        customer: "customer" + userId,
        date: date,
        products: productIds
      });
    }

    // console.log("nosql - creating driver records");
    for (let i = 0; i < orderDates.length; i++) {
      const date = orderDates[i];
      const orders = await db
        .collection("order")
        .find({ date })
        .toArray();
      const warehouseId = randomIntFromInterval(0, warehousesCount);
      await db.collection("drive_record").insertOne({
        _id: "drive_record" + i,
        travel_expenses: randomIntFromInterval(10, 500),
        warehouse: "warehouse" + warehouseId,
        orders
      });
    }
  } catch (e) {
    console.error(e);
  } finally {
    await client.close();
  }
};

module.exports = async multiplier => {
  await generateData(multiplier);
};
