const { Client } = require("pg");
const faker = require("faker");
const seedrandom = require("seedrandom");
const fs = require("fs");

faker.seed(123);
const random = seedrandom(123);
const randomIntFromInterval = (start, end) => {
  return Math.floor(random() * end) + start;
};

const USER_INSERT_QUERY =
  "INSERT INTO public.customer(city, customer_code, zip_code, street) VALUES($1, $2, $3, $4)";
const PRODUCT_INSERT_QUERY =
  "INSERT INTO public.product(product_code, price, profit_margin) VALUES($1, $2, $3)";
const WAREHOUSE_INSERT_QUERY =
  "INSERT INTO public.warehouse(warehouse_code, city, zip_code, street) VALUES($1, $2, $3, $4)";
const ORDER_INSERT_QUERY =
  "INSERT INTO public.order(user_id, date) VALUES($1, $2)";
const ORDER_PRODUCT_INSERT_QUERY =
  "INSERT INTO public.order_product(id_order, id_product) VALUES($1, $2)";
const DRIVE_RECORD_INSERT_QUERY =
  "INSERT INTO public.drive_record(warehouse_id, travel_expenses) VALUES($1, $2)";
const ORDER_DRIVE_RECORD_QUERY =
  "INSERT INTO public.order_drive_record(id_drive_record, id_order) VALUES($1, $2)";

const generateData = async multiplier => {
  const client = new Client({
    user: "dipl",
    host: "localhost",
    database: "diplomka",
    password: "root",
    port: 5432
  });

  const cityAddress = [];
  const orderDates = [];
  const usersCount = 1000 * multiplier;
  const productsCount = 10000 * multiplier;
  const warehousesCount = 10 * multiplier;
  const ordersCount = 5000 * multiplier;
  const driveRecordsCount = 500 * multiplier;
  const cityAddressCount = 10 * multiplier;
  const overDateCount = 50 * multiplier;
  const maxItemsInOrderCount = 50 * multiplier;

  for (let i = 0; i < cityAddressCount; i++) {
    cityAddress.push([faker.address.city(), faker.address.zipCode()]);
  }

  for (let i = 0; i < driveRecordsCount; i++) {
    orderDates.push(faker.date.past());
  }

  try {
    // console.log("sql - creating connection");
    await client.connect();
    // console.log("sql - dropping and creating db");
    await client.query(fs.readFileSync("./createDb.sql").toString());

    // console.log("sql - creating users");
    for (let i = 0; i < usersCount; i++) {
      const cA = cityAddress[randomIntFromInterval(0, cityAddressCount)];
      await client.query(USER_INSERT_QUERY, [
        cA[0],
        "customer" + i,
        cA[1],
        faker.address.streetName()
      ]);
    }

    // console.log("sql - creating products");
    for (let i = 0; i < productsCount; i++) {
      await client.query(PRODUCT_INSERT_QUERY, [
        "product" + i,
        randomIntFromInterval(100, 10000),
        randomIntFromInterval(1, 20)
      ]);
    }

    // console.log("sql - creating warehouses");
    for (let i = 0; i < warehousesCount; i++) {
      const cA = cityAddress[randomIntFromInterval(0, cityAddressCount)];
      await client.query(WAREHOUSE_INSERT_QUERY, [
        "warehouse" + i,
        cA[0],
        cA[1],
        faker.address.streetName()
      ]);
    }

    // console.log("sql - creating orders with products");
    for (let i = 0; i < ordersCount; i++) {
      const userId = randomIntFromInterval(1, usersCount);
      let date =
        orderDates[randomIntFromInterval(0, driveRecordsCount + overDateCount)];
      if (!date) {
        date = faker.date.future();
      }
      await client.query(ORDER_INSERT_QUERY, [userId, date]);
      const orderId = i + 1;
      for (let j = 0; j < randomIntFromInterval(1, maxItemsInOrderCount); j++) {
        const productId = randomIntFromInterval(1, productsCount);
        await client.query(ORDER_PRODUCT_INSERT_QUERY, [orderId, productId]);
      }
    }

    // console.log("sql - creating drive records");
    for (let i = 0; i < orderDates.length; i++) {
      const date = orderDates[i];
      const result = await client.query(
        "SELECT o.id FROM public.order o where o.date = $1",
        [date]
      );
      const warehouseId = randomIntFromInterval(1, warehousesCount);
      await client.query(DRIVE_RECORD_INSERT_QUERY, [
        warehouseId,
        randomIntFromInterval(10, 500)
      ]);
      for (let j = 0; j < result.rows.length; j++) {
        const orderId = result.rows[j].id;
        await client.query(ORDER_DRIVE_RECORD_QUERY, [i + 1, orderId]);
      }
    }
  } catch (e) {
    console.error(e);
  } finally {
    await client.end();
  }
};

module.exports = async multiplier => {
  await generateData(multiplier);
};
